//-----------------------------------------------------------------------
//	File:		$RCSfile: Lab5Testbench.v,v $
//	Version:	$Revision: 1.4 $
//	Desc:		Lab5Testbench
//	Author:		Greg Gibeling
//	Copyright:	Copyright 2003 UC Berkeley
//	This copyright header must appear in all derivative works.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Change Log
//-----------------------------------------------------------------------
//	$Log: Lab5Testbench.v,v $
//	Revision 1.4  2004/10/05 22:40:54  Administrator
//	Fixed bitwidth errors for OpCounter
//	
//	Revision 1.3  2004/09/10 00:11:20  Administrator
//	Fixed test input generator
//	Added output check
//	
//	Revision 1.2  2004/09/09 23:26:02  Administrator
//	Total testbench rewrite
//	Now provides complete Eth2Audio testing
//	
//	Revision 1.1  2004/08/13 01:53:08  SYSTEM
//	Initial Import
//	
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Defines and Constants
//-----------------------------------------------------------------------
`timescale		1 ns/1 ps
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:		Lab5Testbench
//	CUT:		Eth2Audio
//-----------------------------------------------------------------------
module	Lab5Testbench;
	//---------------------------------------------------------------
	//	Parameters
	//---------------------------------------------------------------
	parameter		ethclockfreq =		27000000,
				audioclockfreq =	12288000;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Constants
	//---------------------------------------------------------------
	localparam		ethcycle =		1000000000/ethclockfreq,
				audiocycle =		1000000000/audioclockfreq;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Wires and Regs
	//---------------------------------------------------------------
	wire			EthernetClock, AudioClock;
	reg			EthernetReset, AudioReset;

	wire	[31:0]		DOut, DOutLast;
	wire			OutValid, OutValidLast, OutValidChangeAllowed;
	wire	[2:0]		OutRequestCount;
	wire			OutRequest, OutRequestLast;

	reg	[31:0]		TestFile[0:511];
	reg	[8:0]		TestFilePos;
	reg	[31:0]		Operation;
	reg	[27:0]		OpCounter;
	reg	[11:0]		LinearData;
	reg	[1:0]		InValidEnable;
	reg	[31:0]		DIn;
	reg			InValid, InPacketValid, InPacketInvalid;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Operation Encodings
	//---------------------------------------------------------------
	localparam		OP_Stop =		4'h0,
				OP_FileData =		4'h1,
				OP_LinearData0 =	4'h2,
				OP_EndPacket =		4'h3,
				OP_LinearDataC =	4'h4,
				OP_Wait =		4'h5;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Assigns
	//---------------------------------------------------------------
	assign	OutRequest =			&OutRequestCount[2:1];
	assign	OutRequestLast =		(&OutRequestCount) | (~|OutRequestCount);
	assign	OutValidChangeAllowed =		OutRequestLast;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	CUT:	Eth2Audio
	//---------------------------------------------------------------
	Eth2Audio	CUT(	.DOut(		DOut),
				.OutValid(	OutValid),
				.OutRequest(	OutRequest),
				.EthernetReset(	EthernetReset),
				.EthernetClock(	EthernetClock),
				.AudioReset(	AudioReset),
				.AudioClock(	AudioClock),
				.DIn(		DIn),
				.InValid(	InValid),
				.InPacketValid(	InPacketValid),
				.InPacketInvalid(InPacketInvalid));
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Clock Sources
	//---------------------------------------------------------------
	ClockSource #(ethclockfreq) EthClockGen(.Enable(1'b1), .Clock(EthernetClock));
	ClockSource #(audioclockfreq) AudClockGen(.Enable(1'b1), .Clock(AudioClock));
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Test File Parser
	//---------------------------------------------------------------
	always @ (posedge EthernetClock) begin
		if (EthernetReset) begin
			TestFilePos =					9'h000;
			LinearData =					12'h000;
			OpCounter =					28'h0000000;
			$readmemh("Lab5TestPackets.txt", TestFile);
			InValidEnable =					2'b00;
		end
		else begin
			Operation =					TestFile[TestFilePos];
			InValidEnable =					InValidEnable + 1;
			InValid =					1'b0;
			InPacketValid =					1'b0;
			InPacketInvalid =				1'b0;

			case (Operation[31:28])
				OP_Stop: $stop;
				OP_FileData: begin
					DIn =				TestFile[TestFilePos + OpCounter + 1];
					if (&InValidEnable) begin
						InValid =		1'b1;
						OpCounter =		OpCounter + 1;
						if (OpCounter == Operation[27:0]) begin
							OpCounter =	28'h0000000;
							TestFilePos =	TestFilePos + Operation[27:0] + 1;
						end
					end
				end
				OP_LinearData0: begin
					if (~|OpCounter) LinearData =	12'h000;
					DIn =				{4'h2, LinearData, 4'h1, LinearData};
					if (&InValidEnable) begin
						InValid =		1'b1;
						OpCounter =		OpCounter + 1;
						LinearData =		LinearData + 1;
						if (OpCounter == Operation[27:0]) begin
							OpCounter =	28'h0000000;
							TestFilePos =	TestFilePos + 1;
						end
					end
				end
				OP_EndPacket: begin
					if (&InValidEnable) begin
						OpCounter =		28'h0000000;
						TestFilePos =		TestFilePos + 1;
						InPacketValid =		~TestFile[TestFilePos][0];
						InPacketInvalid =	TestFile[TestFilePos][0];
					end
				end
				OP_LinearDataC: begin
					DIn =				{4'h2, LinearData, 4'h1, LinearData};
					if (&InValidEnable) begin
						InValid =		1'b1;
						OpCounter =		OpCounter + 1;
						LinearData =		LinearData + 1;
						if (OpCounter == Operation[27:0]) begin
							OpCounter =	28'h0000000;
							TestFilePos =	TestFilePos + 1;
						end
					end
				end
				OP_Wait: begin
					DIn =				32'hXXXXXXXX;
					OpCounter =			OpCounter + 1;
					if (OpCounter == Operation[27:0]) begin
						OpCounter =		28'h0000000;
						TestFilePos =		TestFilePos + 1;
					end
				end
				default: DIn =				32'hXXXXXXXX;
			endcase
		end
	end
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Test Stimulus
	//---------------------------------------------------------------
	initial begin
		EthernetReset =			1'b0;
		#(ethcycle);
		EthernetReset =			1'b1;
		#(ethcycle * 16);
		EthernetReset =			1'b0;
	end

	initial begin
		AudioReset =			1'b0;
		#(audiocycle);
		AudioReset =			1'b1;
		#(audiocycle * 16);
		AudioReset =			1'b0;
	end
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Output Request Counter
	//---------------------------------------------------------------
	Counter		ORCnt(	.Clock(		AudioClock),
				.Reset(		AudioReset),
				.Set(		1'b0),
				.Load(		1'b0),
				.Enable(	1'b1),
				.In(		3'bxxx),
				.Count(		OutRequestCount));
	defparam	ORCnt.width =		3;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Output Valid Register
	//---------------------------------------------------------------
	Register	OVReg(	.Clock(		AudioClock),
				.Reset(		AudioReset),
				.Set(		1'b0),
				.Enable(	1'b1),
				.In(		OutValid),
				.Out(		OutValidLast));
	defparam	OVReg.width =		1;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Output Valid Check
	//---------------------------------------------------------------
	always @ (posedge AudioClock) begin
		if (~OutValidChangeAllowed & (OutValid ^ OutValidLast)) $display("ERROR: OutValid changed even when OutRequest wasn't signalled!! (time: %t)", $time);
	end
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Last Output Register
	//---------------------------------------------------------------
	Register	DOLReg(	.Clock(		AudioClock),
				.Reset(		AudioReset),
				.Set(		1'b0),
				.Enable(	OutValid),
				.In(		DOut),
				.Out(		DOutLast));
	defparam	DOLReg.width =		32;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Print Output Changes
	//---------------------------------------------------------------
	always @ (posedge AudioClock) begin
		if (OutRequestLast & OutValid & ((DOut[11:0] != (DOutLast[11:0] + 1)) | (DOut[27:16] != (DOutLast[27:16] + 1)))) $display("NOTE: DOut went from %h to %h. (time: %t)", DOutLast, DOut, $time);
	end
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------