//-----------------------------------------------------------------------
//	File:		$RCSfile: Reverse.v,v $
//	Version:	$Revision: 1.4 $
//	Desc:		Bit Reversal Module (a fancy wire)
//	Author:		Greg Gibeling
//	Copyright:	Copyright 2003 UC Berkeley
//	This copyright header must appear in all derivative works.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Change Log
//-----------------------------------------------------------------------
//	$Log: Reverse.v,v $
//	Revision 1.4  2004/08/10 16:43:01  SYSTEM
//	Reformatted
//	
//	Revision 1.3  2004/07/29 20:50:17  Administrator
//	Updated Comments
//	
//	Revision 1.2  2004/06/17 18:59:57  Administrator
//	Added Proper Headers
//	Updated Parameters and Constants
//	General Housekeeping
//	
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:	Reverse
//	Desc:	Parameterized bit reversal module
//		Essentially this module is just a very complex set of
//		wires, all it does it reverse the bits in a bus.
//		A "group" is a group of "chunks" that should be reversed,
//		"set" is the number of "chunks" per "group".
//		This module will reverse the order of the "chunks" within
//		each "group".
//	Params:	width:	This sets the input and output bus width of the
//			module
//		chunk:	This is the size of a block of wires which should
//			be kept in order.  The default is 1 meaning each
//			wire should be treated seperately.
//		set:	The number of chunks in a set, the default is the
//			bitwidth of the input bus, meaning that the whole
//			input bus is treated as a set.
//	Ex:	(32,1,32) will reverse the bit order of a 32bit bus.
//		(32,1,8) will reverse the MSb/LSb order of the bytes in a
//		32bit bus.
//		(32,4,2) will reverse the MSNibble/LSNibble of each byte
//		in a 32bit bus.
//-----------------------------------------------------------------------
module	Reverse(In, Out);
	//---------------------------------------------------------------
	//	Parameters
	//---------------------------------------------------------------
	parameter		width =			32,
				chunk =			1,
				set =			width;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Constants
	//---------------------------------------------------------------
	localparam		group =			chunk * set;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	I/O
	//---------------------------------------------------------------
	input	[width-1:0]	In;
	output	[width-1:0]	Out;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	A Complicated Wire
	//---------------------------------------------------------------
	genvar			i;
	generate for(i = 0; i < width; i = i + 1) begin:reverse
		assign Out[i] = In[((set - 1 - ((i % group) / chunk)) * chunk) + ((i % group) % chunk) + ((i / group) * group)];
	end endgenerate
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------